
import os

sectors_log_dir = os.path.abspath("./sectors.logs") if os.name != 'nt' else os.path.abspath(".\\Calibnet\\sectors.logs")

#################
# line analysis #
#################
# in:  [event;sealing.SectorStartCC]
# out: SectorPreCommit1
def line2event(line):
    prefix="event;sealing."
    s = line.find(prefix) + len(prefix)
    return line[s:line.find("]")]

# in:  0.	2021-06-21 23:52:25 +0800 CST:	[event;sealing.SectorStartCC]	{"User":{"ID":30,"SectorType":8}}
# out: ['0.', '2021-06-21 23:52:25 +0800 CST:', '[event;sealing.SectorStartCC]', '{"User":{"ID":30,"SectorType":8}}']
def line2list(line):
    return line.split('\t')

import datetime
import time

# in:  2021-06-21 23:52:25 +0800 CST:
# out: datetime.datetime(2021, 6, 21, 23, 52, 25)
def string2datetime(st):
    return datetime.datetime.strptime(st, "%Y-%m-%d %H:%M:%S +0800 CST:")

#################
# file analysis #
#################

events=[
        "SectorStartCC",
        "SectorPacked",
        "SectorTicket",
        "SectorPreCommit1",
        "SectorPreCommit2",
        "SectorPreCommitBatch",
        "SectorPreCommitBatchSent",
        "SectorPreCommitLanded",
        "SectorSeedReady",
        "SectorCommitted",
        "SectorSubmitCommitAggregate",
        "SectorCommitAggregateSent",
        "SectorProving",
        "SectorFinalized",
        "SectorSealPreCommit1Failed",
        "SectorRetrySealPreCommit1",
]

def file2events(logfile):
    with open(logfile,"r") as f:
        lines=f.readlines()
        return lines2events(lines)

def lines2events(lines):
    sector_report={}
    previous=0
    for line in lines:
        items=line2list(line.rstrip("\n"))
        if len(items) < 4: continue
        now=string2datetime(items[1])
        if "SectorStartCC" in items[2]: previous=now
        sector_report[line2event(items[2])] = {
                                                "order":items[0].rstrip("."),   
                                                "time":now,
                                                "log":items[3],
                                                "period":now-previous
                                                }
        previous=now
    return sector_report

stages = ["SectorPacked","SectorPreCommit1","SectorPreCommit2","SectorSeedReady","SectorCommitted"]

def printReport(report):
    if "SectorStartCC" in report.keys(): print("SectorStartCC     {0}".format(report["SectorStartCC"]["time"]))
    if "SectorPacked" in report.keys():  print("SectorPacked:     {0}".format(report["SectorPacked"]["period"].strftime("%H:%M:%S")))
    if "SectorPreCommit1" in report.keys(): print("SectorPreCommit1: {0}".format(report["SectorPreCommit1"]["period"].strftime("%H:%M:%S")))
    if "SectorPreCommit2" in report.keys(): print("SectorPreCommit2: {0}".format(report["SectorPreCommit2"]["period"].strftime("%H:%M:%S")))
    # if "SectorSeedReady" in report.keys(): print("SectorSeedReady:  {0}".format(report["SectorSeedReady"]["period"])
    if "SectorCommitted" in report.keys(): print("SectorCommitted:  {0}".format(report["SectorCommitted"]["period"].strftime("%H:%M:%S")))
    if "SectorFinalized" in report.keys(): print("SectorFinalized:  {0}".format(report["SectorFinalized"]["time"]))

#################
#  dir analysis #
#################

def makeReport(start_id,end_id,issectorprinted=False):
    reports=[]
    for idx in range(start_id,end_id):
        logfile=os.path.join(sectors_log_dir,"%d.log"%idx)
        issectorprinted: print(logfile)
        if not os.path.exists(logfile): continue
        sector_report=file2events(logfile)
        printReport(sector_report)
        reports.append(sector_report)
        
    return reports

from string import Template

class DeltaTemplate(Template):
    delimiter = "%"

def strfdelta(tdelta, fmt="%D:%H:%M:%S"):
    d = {"D": tdelta.days}
    hours, rem = divmod(tdelta.seconds, 3600)
    minutes, seconds = divmod(rem, 60)
    d["H"] = '{:02d}'.format(hours)
    d["M"] = '{:02d}'.format(minutes)
    d["S"] = '{:02d}'.format(seconds)
    t = DeltaTemplate(fmt)
    return t.substitute(**d)

def analyzeReport(reports):
    print("*******************************************")
    for stage in stages:        
        ## max, min, mean
        max_=datetime.timedelta(0)
        min_=datetime.timedelta(days=10)
        mean_=datetime.timedelta(0)
        ## initialize
        sum=datetime.timedelta(0)
        cnt=0
        for report in reports:
            if stage not in report.keys(): continue
            if "period" not in report[stage].keys(): continue
            if report[stage]["period"] > max_: max_=report[stage]["period"]
            if report[stage]["period"] < min_: min_=report[stage]["period"]
            sum+= report[stage]["period"]
            cnt+=1
        mean_=sum/cnt if cnt!=0 else sum
        print("[%s]"%stage)
        print("MIN:  {0}".format(min_))
        print("MAX:  {0}".format(max_))
        print("MEAN: {0}".format(strfdelta(mean_)))
        print("*******************************************")

    ## calc
    start=reports[0]["SectorStartCC"]["time"]
    end=start
    for report in reports:
        last=end
        if "SectorStartCC" in report.keys(): last=report["SectorStartCC"]["time"]
        if "SectorFinalized" in report.keys(): last=report["SectorFinalized"]["time"]
        if last>end: end=last
    duration=(end-start).total_seconds()/(24*3600.0)#days
    sector_size = 32#GiB
    _as_terabyte = sector_size/1000.0
    p1_cnt,p2_cnt,c2_cnt,fin_cnt=0,0,0,0
    for report in reports:
        if "SectorPreCommit1" in report.keys(): p1_cnt+=1
        if "SectorPreCommit2" in report.keys(): p2_cnt+=1
        if "SectorCommitted" in report.keys(): c2_cnt+=1
        if "SectorFinalized" in report.keys(): fin_cnt+=1
    print("DURATION: {0}  START: {1}, FINISH: {2}".format("%.1f days"%duration,start,end))
    print("SECTOR SIZE: %d(GiB)"%sector_size)
    print("PC1: %.2fT, PC2: %.2fT, C: %.2fT, FIN: %.2fT"%(\
                p1_cnt*_as_terabyte/duration,\
                p2_cnt*_as_terabyte/duration,\
                c2_cnt*_as_terabyte/duration,\
                fin_cnt*_as_terabyte/duration))

from lotusops.cli.lotuspledge import runscript

# lotusspeed miner 110 300
# argv=[110,300]
# lotusspeed miner 110
# arv [110]
# lotusspeed miner detail
# argv=['detail']
def AnalyzeSectorLogs(argv):
    
    try:
        output=runscript("lotus-miner sectors list",isansible=False)
        candidates = [line.split()[0] for line in output[1:]]
        sector_ids = [int(candidate) for candidate in candidates if candidate.isdigit()]
    except:
        return
    # start:end
    start = int(argv[0]) if len(argv)>0 and argv[0].isdigit() else sector_ids[0]
    end = int(argv[1]) if len(argv)>1 and argv[1].isdigit() else sector_ids[-1]
    issectorprinted=True if len(argv)>0 and not argv[0].isdigit() else False
    # make report
    reports=[]
    for id in sector_ids:
        if id < start: continue
        if id > end: break
        sector_log=runscript("lotus-miner sectors status --log %d"%id,isansible=False)
        sector_report=lines2events(sector_log)
        if issectorprinted: printReport(sector_report)
        reports.append(sector_report)
    # analyze report
    analyzeReport(reports)


if __name__ == "__main__":
    sectors=[]
    for f in os.listdir(sectors_log_dir):
        sectors.append(int(f.rstrip(".log")))
    sectors.sort()
    # print(sectors)
    first_id=min(sectors)
    last_id=max(sectors)
    # input start_sector_id, end_sector_id
    reports=makeReport(first_id,last_id)
    print("###################################################")
    analyzeReport(reports)


import os
import sys

msg_help = "\n\
            \n======== lotuspledge =========\
            \n\
            \nDescription: \
            \n            A lotus auto-pleding assitant tool for implementing auto-pledge more smartly with inspecting workers.\
            \n            It repeats to pledge a sector in a specific interval as long as any worker is affordable for the task. \
            \n            It's a capsulation tool based on lotus-miner & shell scripts. Make sure lotus-miner is already installed.\
            \nUsage: \
            \n       lotuspledge <time-interval> <worker.ip.lst>\
            \nExamples: \
            \n       lotuspledge 1m '172.26.48.134|172.26.48.135'\
            \n       lotuspledge 90 /root/workers.lst\
            \n       lotuspledge inspect '172.26.48.134|172.26.48.135'"


def lotuspledge():
    if  len(sys.argv) < 3 or \
        any(any(arg.startswith(mark) for mark in ["-h","--h","help"]) for arg in sys.argv) or \
        not load_ips(sys.argv[2]): 
            return msg_help
    autopledge(sys.argv[1],sys.argv[2])

import subprocess
import string

msg_workers_needed="The list of precommit workers connected to the miner should be indicated using file or string."
msg_ansible_needed="this action needs anible installed."
msg_ansible_needs_workers="ansible needs workers' ip. It must be added in /etc/ansible/hosts"

def load_ips(iplst):
    ips=[]
    if os.path.isfile(iplst):
        with open(iplst,'r') as f:
            for line in f.readlines():
                ips.append(line.rstrip("\n"))
    else:
        ips=iplst.split("|")
    return ips

def interpret(arg):
    m2s=60
    h2s=m2s*60
    d2s=24*h2s
    try:
        output = {
            'm':m2s*int(arg[:-1]),
            'h':h2s*int(arg[:-1]),
            'd':d2s*int(arg[:-1]),
            's':d2s*int(arg),
        }.get([arg[-1]],int(arg))
    except:
        output = 60
    if output < 60: output=60
    return output

ansible_hosts_path='/etc/ansible/hosts'

def getAnsibleNames(ips):
    ansible_names={}
    with open(ansible_hosts_path,'r') as f:
        for line in f.readlines():
            for ip in ips:
                if ip in line: ansible_names[ip]=line.rstrip("\n").split()[0]
    return ansible_names

def runscript(script,isansible=True):
    
    if sys.version_info[0]<3:
        import commands
        result = commands.getstatusoutput(script).split("\n")
    else:
        import subprocess
        result = subprocess.getoutput(script).split("\n")
    if not isansible: return result
    # just handling one node
    # multiple node mode not implemented yet
    # 'test1 | SUCCESS | rc=0 >>'
    # 'test2 | CHANGED | rc=0 >>'
    non_ansible_start_index=1
    for idx,line in enumerate(result):
        splited=line.split()
        if splited[0] == script.split()[1] and \
            all('|'== splited[i] for i in [1,3]) and\
            splited[4] == "rc=0":
            non_ansible_start_index=idx+1;break
    #
    return  result[non_ansible_start_index:]

PLEDGEING_INTEVERAL=150#s

# in: --resourceUsage 0.4
# in: --resourceUsage=0.4
# out 0.4
def extractValue(line,key="--resourceUsage"):
    splited=line.split()
    for idx,slice in enumerate(splited):
        if key not in slice: continue
        if key == slice: return splited[idx+1]
        if "=" in slice: return slice.split("=")[1]
    return None

import datetime

# export LOTUS_MINER_PATH=
# 
def autopledge(interval,iplst):
    # load ips
    ips=load_ips(iplst)
    if len(ips) < 1: return msg_workers_needed
    # map(ip,ansible_name)
    ansible_names=getAnsibleNames(ips)
    if len(ansible_names) < 1: return msg_ansible_needs_workers
    INSPECT_INTERVAL = interpret(interval)
    while 1:
        print("Time: ",datetime.datetime.now())
        ip_process_env_tree={}
        # inquiry
        # {'172.26.48.134':{
        #                   '11111': {LOTUS_WORKER_PATH:/filecoin1/lotusworker5}},
        #                   22222': {LOTUS_WORKER_PATH:/filecoin1/lotusworker1}
        #                   }
        # }
        # make (ip,procs)
        for ip in ips:
            ip_process_env_tree[ip]={}
            script="ansible workername -m shell -a 'ps -ef|grep lotus'".replace('workername',ansible_names[ip])
            procs = list(filter(None,[line \
                                    for line in runscript(script) \
                                    if 'lotus-worker' in line]))
            resource_for_precommit = 0.0
            ip_process_env_tree[ip]["RESOURCE_USAGE(PRECOMMIT)"] = 0.0
            ip_process_env_tree[ip]['CPU'] = {}
            ip_process_env_tree[ip]['MEM'] = {}
            for proc in procs:
                envdict={}
                pid=proc.split()[1]
                # tasks for process
                envdict["TYPE"] = "COMMIT" if "--precommit1=false" in proc else "PRECOMMIT"
                # resourceUsage
                if "resourceUsage" in proc:
                    try: envdict["RESOURCE_USAGE"] = float(extractValue(proc,key="--resourceUsage"))
                    except: envdict["RESOURCE_USAGE"] = 1.0
                # (process,LOTUS_WORKER_PATH)
                script="ansible {0} -m shell -a 'strings /proc/{1}/environ'".format(ansible_names[ip],pid)
                envs=list(filter(None,[line\
                                    for line in runscript(script) \
                                    if any(filter in line for filter in ['FIL','LOTUS','CPU','CUDA','TMP'])]))
                # environment variables added to tree
                for env in envs:
                    k,v=env.split("=")
                    envdict[k]=v
                ip_process_env_tree[ip][pid]=envdict
                # resource usage
                if envdict["TYPE"] == "PRECOMMIT" and "RESOURCE_USAGE" in envdict.keys():
                    resource_for_precommit += envdict["RESOURCE_USAGE"]
            # resource usage
            ip_process_env_tree[ip]["RESOURCE_USAGE(PRECOMMIT)"] = 1.0 if resource_for_precommit == 0 else resource_for_precommit

        # check availability for pledging    
        pledgeable_cnt = chkavailable(ip_process_env_tree)
        if any("inspect" in arg for arg in sys.argv): break
        import time
        while pledgeable_cnt>0: 
            print(runscript("lotus-miner sectors pledge",isansible=False))
            pledgeable_cnt-=1
            time.sleep(PLEDGEING_INTEVERAL)
        time.sleep(INSPECT_INTERVAL)

PLEDGE_MARGIN = 1

def chkavailable(ip_process_env_tree):
    storage_per_sector_precommit,storage_per_sector_commit=(32+32+453),(32+32)
    cpus_per_sector = 3
    mem_per_sector = 64
    storage_per_sector_as_kb_for_precommit=storage_per_sector_precommit*2**20#KByte
    ansible_names = getAnsibleNames(ip_process_env_tree.keys())
    # inspect miner
    script='lotus-miner sealing jobs|egrep "AP|PC1|PC2|C1|C2|GET"'
    jobs=list(filter(None,runscript(script,isansible=False)))
    jstats=set([job.split()[4] for job in jobs if not job.split()[4].isdigit()])
    jstats_precommit="AP|PC1|PC2".split("|")
    sectors_cnt_assigned = len([job for job in jobs if any(stat in job.split()[4] for stat in jstats_precommit)])
    workers = {}
    for job in jobs:
        try: wid= job.split()[2]
        except: continue
        if len(wid) !=8 or not all(c in string.hexdigits for c in wid): continue
        if wid not in workers.keys():
            workers[wid] = [job.split()[1]]
        else: workers[wid].append(job.split()[1])
    script='lotus-miner sectors list|egrep "Packing|PreCommit1|PreCommit2|WaitSeed|CommitWait|Committing|FinalizeSector|Removing|RecoveryTimeout"'
    sectors=list(filter(None,runscript(script,isansible=False)))
    sstats=set([sector.split()[1] for sector in sectors if not sector.split()[1].isdigit()])
    sstats_precommit="Packing|PreCommit1|PreCommit2".split("|")
    sectors_cnt_queue = len([sector for sector in sectors if any(stat in sector.split()[1] for stat in sstats_precommit)])
    sectors_cnt=max(sectors_cnt_queue,sectors_cnt_assigned)
    print("#######################################################")
    print("MINER REPORT:")
    print("              Queue(%s)"%",".join(["{0}:{1}".format(stat,len([sector for sector in sectors if stat in sector ])) for stat in sstats]))
    print("           Assigned(%s)"%",".join(["{0}:{1}".format(stat,len([job for job in jobs if stat in job ])) for stat in jstats]))

    # inspect workers
    print("#######################################################")
    print("WORKER REPORT: ")
    cached_sectors_cnt,pledgeable_sectors_cnt = 0,0
    for ip,procs in ip_process_env_tree.items():
        
        action = "ansible %s -m shell -a"%ansible_names[ip]
        
        ip_used_mem,ip_total_mem=0,0
        ip_used_cpus,ip_total_cpus=0,0
        ip_sector_cnt,ip_used_storage,ip_total_storage=0,0,0
        # MEM
        output=runscript("ansible workername -m shell -a 'free'".\
                        replace('workername',ansible_names[ip]))[1].split()
        try:
            ip_total_mem = int(float(output[1])/2**20)
            ip_total_used_mem = int(float(output[2])/2**20)
        except: continue
        # CPU
        output=runscript("ansible workername -m shell -a 'nproc --all'".\
                        replace('workername',ansible_names[ip]))[0]
        try: ip_total_cpus=int(output)
        except: continue
        # USED
        output=runscript("ansible workername -m shell -a 'ps -eo pid,%mem,%cpu,comm --sort=-%cpu'".replace('workername',ansible_names[ip]))[1:]
        ip_proc_usage={line.split()[0]:{"mem":line.split()[1],"cpu":line.split()[2],"pname":line.split()[3]} for line in output}
        ip_total_used_cpus=int(sum(float(value["cpu"]) for value in ip_proc_usage.values())/100)
        ip_total_used_mem=int(max(sum(float(value["mem"]) for value in ip_proc_usage.values())/100*ip_total_mem,ip_used_mem))
        # STORAGE FOR PreCommit
        lotus_disks=[]
        ip_process_env_tree[ip]['STORAGE'] = {
                                                'PATH':{}, \
                                                'PLEDGE_USEABLE(CNT)':0 \
                                            }
        for pid,env in procs.items():
            if any(c not in string.digits for c in pid) or\
               not isinstance(env,dict): continue
            if not env['LOTUS_WORKER_PATH']: continue

            storage_root_path=env['LOTUS_WORKER_PATH'].split("/")[1]
            if storage_root_path not in lotus_disks: 
                lotus_disks.append(storage_root_path)
            # STORAGE, $LOTUS_WORKER_PATH/cache
            # ansible workername -m shell -a 'du $LOTUS_WORKER_PATH/cache -hd1'
            script="%s 'du %s/cache -d1'"%(action,env['LOTUS_WORKER_PATH'])
            sids=list(filter(None,[int(line.split("-")[-1]) \
                            for line in runscript(script) if all(mark in line for mark in ["s-t","cache"])]))
            sids.sort()
            
            ip_process_env_tree[ip][pid]['WORKER'] = {}
            ip_process_env_tree[ip][pid]['WORKER']['CACHED'] = ip_process_env_tree[ip][pid]['WORKER']['RUNNING'] = sids
            ip_process_env_tree[ip][pid]['WORKER']['ID'] = "000000"
            for wid,_sids in workers.items():
                __sids=[int(sid) for sid in _sids];__sids.sort()
                if any(sid in sids for sid in __sids):
                    ip_process_env_tree[ip][pid]['WORKER']['ID'] = wid
                    ip_process_env_tree[ip][pid]['WORKER']['RUNNING'] = __sids
                    break

            ip_process_env_tree[ip][pid]['WORKER']['MISMATCH'] = set(ip_process_env_tree[ip][pid]['WORKER']['CACHED'])^\
                                                                 set(ip_process_env_tree[ip][pid]['WORKER']['RUNNING'])
            
            ip_process_env_tree[ip][pid]['WORKER']['CACHED']=",".join([str(sid) for sid in ip_process_env_tree[ip][pid]['WORKER']['CACHED']])
            ip_process_env_tree[ip][pid]['WORKER']['RUNNING']=",".join([str(sid) for sid in ip_process_env_tree[ip][pid]['WORKER']['RUNNING']])
            ip_process_env_tree[ip][pid]['WORKER']['MISMATCH']=",".join([str(sid) for sid in ip_process_env_tree[ip][pid]['WORKER']['MISMATCH']])
            ip_process_env_tree[ip][pid]['CACHED_SECTOR_CNT'] = len(sids)
            # STORAGE, $LOTUS_WORKER_PATH
            # ansible workername -m shell -a 'du $LOTUS_WORKER_PATH -d1'
            script="%s 'du %s -d1'"%(action,env['LOTUS_WORKER_PATH'])
            proc_used_storage=int(runscript(script)[-1].split()[0])
            ip_process_env_tree[ip][pid]['STORAGE'] = int(proc_used_storage/2**20)
            # CPU, MEM
            proc_mem_usage=int(float(ip_proc_usage[pid]["mem"])/100*ip_total_mem)
            proc_cpu_usage=int(float(ip_proc_usage[pid]["cpu"])/100)
            ip_process_env_tree[ip][pid]['MEM']=proc_mem_usage
            ip_process_env_tree[ip][pid]['CPU']=proc_cpu_usage
            # summing up
            ip_used_mem += ip_process_env_tree[ip][pid]['MEM']
            ip_used_cpus += ip_process_env_tree[ip][pid]['CPU']
            ip_sector_cnt += ip_process_env_tree[ip][pid]['CACHED_SECTOR_CNT']
            ip_used_storage += ip_process_env_tree[ip][pid]['STORAGE']
            
            # excluding c2 for storage

            if storage_root_path not in ip_process_env_tree[ip]['STORAGE']['PATH'].keys():
                ip_process_env_tree[ip]['STORAGE']['PATH'][storage_root_path] = {}
                ip_process_env_tree[ip]['STORAGE']['PATH'][storage_root_path]["TYPE"] = []
                ip_process_env_tree[ip]['STORAGE']['PATH'][storage_root_path]['PRECOMMIT'] = {}
                ip_process_env_tree[ip]['STORAGE']['PATH'][storage_root_path]['PRECOMMIT']['USED'] = 0
                ip_process_env_tree[ip]['STORAGE']['PATH'][storage_root_path]['PRECOMMIT']['CACHED_SECTOR_CNT'] = 0
                ip_process_env_tree[ip]['STORAGE']['PATH'][storage_root_path]['COMMIT'] = {}
                ip_process_env_tree[ip]['STORAGE']['PATH'][storage_root_path]['COMMIT']['USED'] = 0
                ip_process_env_tree[ip]['STORAGE']['PATH'][storage_root_path]['COMMIT']['CACHED_SECTOR_CNT'] = 0

            ip_process_env_tree[ip]['STORAGE']['PATH'][storage_root_path][env["TYPE"]]['USED'] +=  ip_process_env_tree[ip][pid]['STORAGE']
            ip_process_env_tree[ip]['STORAGE']['PATH'][storage_root_path][env["TYPE"]]['CACHED_SECTOR_CNT'] += ip_process_env_tree[ip][pid]['CACHED_SECTOR_CNT']

            ip_process_env_tree[ip]['STORAGE']['PATH'][storage_root_path]["TYPE"].append(env["TYPE"])


        # ansible workername -m shell -a 'df'
        df_out=list(filter(None,[line
                                 for line in runscript(action+" 'df'") \
                                 if any(disk in line for disk in lotus_disks)]))

        for disk in lotus_disks:
            ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['SIZE'] = int([int(line.split()[1]) for line in df_out if disk in line][0]/2**20)
            ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['USED'] = int([int(line.split()[2]) for line in df_out if disk in line][0]/2**20)
            # cached
            ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['PRECOMMIT']['CACHED'] = \
                    ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['PRECOMMIT']['CACHED_SECTOR_CNT'] * storage_per_sector_precommit
            ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['COMMIT']['CACHED'] = \
                    ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['COMMIT']['CACHED_SECTOR_CNT'] * storage_per_sector_commit
            ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['CACHED'] = \
                ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['PRECOMMIT']['CACHED'] +\
                ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['COMMIT']['CACHED']
            # non_lotus usage
            ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['NON_LOTUS_USED'] = \
                    ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['USED']-\
                    ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['PRECOMMIT']['USED']-\
                    ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['COMMIT']['USED']
            ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['NON_LOTUS_USED']=max(0,ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['NON_LOTUS_USED'])
            # lotus usable
            ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['LOTUS_USEABLE'] = \
                    ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['SIZE'] -\
                    ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['NON_LOTUS_USED']
            ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['LOTUS_USEABLE'] = max(0,ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['LOTUS_USEABLE'])
            # pledgeable
            ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['PLEDGE_USEABLE'] = \
                ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['LOTUS_USEABLE'] -\
                max(ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['COMMIT']['CACHED'],\
                    ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['COMMIT']['USED'])\
                if  any(job == "PRECOMMIT" for job in ip_process_env_tree[ip]['STORAGE']['PATH'][disk]["TYPE"]) \
                else 0
            ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['PLEDGE_USEABLE']=max(0,ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['PLEDGE_USEABLE']-PLEDGE_MARGIN)
            ip_process_env_tree[ip]['STORAGE']['PLEDGE_USEABLE(CNT)'] += int(ip_process_env_tree[ip]['STORAGE']['PATH'][disk]['PLEDGE_USEABLE']/storage_per_sector_precommit)
        # cpu
        ip_process_env_tree[ip]['CPU']['TOTAL'] = ip_total_cpus
        ip_process_env_tree[ip]['CPU']['LOTUS_USED'] = ip_used_cpus
        ip_process_env_tree[ip]['CPU']['NON_LOTUS_USED'] = ip_total_used_cpus-ip_used_cpus
        ip_process_env_tree[ip]['CPU']['LOTUS_USEABLE(CNT)'] = int(max(ip_total_cpus-ip_process_env_tree[ip]['CPU']['NON_LOTUS_USED'],0)/cpus_per_sector)
        ip_process_env_tree[ip]['CPU']['LOTUS_USEABLE(CNT)'] = int(min(ip_process_env_tree[ip]["RESOURCE_USAGE(PRECOMMIT)"],1.0)*ip_process_env_tree[ip]['CPU']['LOTUS_USEABLE(CNT)'])
        # mem
        ip_process_env_tree[ip]['MEM']['TOTAL'] = ip_total_mem
        ip_process_env_tree[ip]['MEM']['LOTUS_USED'] = ip_used_mem
        ip_process_env_tree[ip]['MEM']['NON_LOTUS_USED'] = ip_total_used_mem-ip_used_mem
        ip_process_env_tree[ip]['MEM']['LOTUS_USEABLE(CNT)'] = int(max(ip_total_mem-ip_process_env_tree[ip]['MEM']['NON_LOTUS_USED'],0)/mem_per_sector)
        ip_process_env_tree[ip]['MEM']['LOTUS_USEABLE(CNT)'] = int(min(ip_process_env_tree[ip]["RESOURCE_USAGE(PRECOMMIT)"],1.0)*ip_process_env_tree[ip]['MEM']['LOTUS_USEABLE(CNT)'])
        # affordability check
        pledgeable_sectors_cnt+=min(ip_process_env_tree[ip]['STORAGE']['PLEDGE_USEABLE(CNT)'],\
                                ip_process_env_tree[ip]['CPU']['LOTUS_USEABLE(CNT)'],\
                                ip_process_env_tree[ip]['MEM']['LOTUS_USEABLE(CNT)'])
        # cached sector cnt
        cached_sectors_cnt+=ip_sector_cnt

    import json; print(json.dumps(ip_process_env_tree, indent=4, sort_keys=True))
    print("PLEDEABLE_CNT: {0}, MINER_SECTOR_CNT: {1}, WORKER_CACHED_SECTOR_CNT: {2}".format(pledgeable_sectors_cnt,sectors_cnt,cached_sectors_cnt))
    return pledgeable_sectors_cnt-sectors_cnt



# cat /filecoin/worker.log |grep commit|egrep -v "validate|seal_commit"
# {"level":"info","ts":"2021-07-26T06:42:40.492+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T06:42:40.492+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(1)"}
# {"level":"info","ts":"2021-07-26T06:44:00.190+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T06:44:00.190+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(2)"}
# {"level":"info","ts":"2021-07-26T06:47:37.908+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T06:47:37.908+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(33)"}
# {"level":"info","ts":"2021-07-26T06:47:44.269+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T06:47:44.269+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(24)"}
# {"level":"info","ts":"2021-07-26T06:47:46.253+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T06:47:46.253+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(42)"}
# {"level":"info","ts":"2021-07-26T06:48:01.563+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T06:48:01.563+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(68)"}
# {"level":"info","ts":"2021-07-26T06:48:06.264+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T06:48:06.264+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(59)"}
# {"level":"info","ts":"2021-07-26T06:48:07.759+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T06:48:07.759+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(6)"}
# {"level":"info","ts":"2021-07-26T06:51:53.697+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T06:51:53.697+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(15)"}
# {"level":"info","ts":"2021-07-26T06:51:53.704+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T06:51:53.704+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(77)"}
# {"level":"info","ts":"2021-07-26T06:51:53.705+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T06:51:53.705+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(50)"}
# {"level":"info","ts":"2021-07-26T07:00:05.507+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T07:00:05.509+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(81)"}
# {"level":"info","ts":"2021-07-26T09:43:17.874+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(1)"}
# {"level":"info","ts":"2021-07-26T09:43:17.877+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T09:43:18.067+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T09:43:18.070+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T09:52:19.388+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(2)"}
# {"level":"info","ts":"2021-07-26T09:52:19.388+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T09:54:12.837+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(42)"}
# {"level":"info","ts":"2021-07-26T09:54:12.837+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T09:55:32.791+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(6)"}
# {"level":"info","ts":"2021-07-26T09:55:32.791+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T09:56:33.704+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(15)"}
# {"level":"info","ts":"2021-07-26T09:56:33.704+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T09:56:50.984+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(59)"}
# {"level":"info","ts":"2021-07-26T09:56:50.984+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T09:57:15.841+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(24)"}
# {"level":"info","ts":"2021-07-26T09:57:15.841+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T09:57:47.256+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(33)"}
# {"level":"info","ts":"2021-07-26T09:57:47.256+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T09:59:20.532+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(50)"}
# {"level":"info","ts":"2021-07-26T09:59:20.532+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T10:00:40.759+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(68)"}
# {"level":"info","ts":"2021-07-26T10:00:40.759+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T10:01:41.059+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T10:01:41.059+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T10:01:41.132+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T10:01:41.132+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T10:03:32.768+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(77)"}
# {"level":"info","ts":"2021-07-26T10:03:32.768+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T10:06:49.621+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(81)"}
# {"level":"info","ts":"2021-07-26T10:06:49.621+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T10:17:51.254+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T10:17:51.254+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T10:17:51.321+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T10:17:51.321+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T10:32:33.939+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T10:32:33.939+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T10:32:34.001+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T10:32:34.001+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T10:48:24.629+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T10:48:24.629+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T10:48:24.705+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T10:48:24.705+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T11:03:24.927+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T11:03:24.927+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T11:03:24.996+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T11:03:24.996+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T11:17:54.070+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T11:17:54.070+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T11:17:54.158+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T11:17:54.158+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T11:32:51.611+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T11:32:51.611+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T11:32:51.672+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T11:32:51.672+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T11:48:10.747+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T11:48:10.747+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T11:48:10.813+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T11:48:10.813+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T12:02:45.860+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T12:02:45.860+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T12:02:45.924+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T12:02:45.925+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T12:02:54.548+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T12:02:54.549+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(89)"}
# {"level":"info","ts":"2021-07-26T12:15:09.848+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T12:15:09.848+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(83)"}
# {"level":"info","ts":"2021-07-26T12:19:33.138+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T12:19:33.138+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T12:19:33.204+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T12:19:33.205+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T12:30:47.774+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:1295","msg":"generate_data_commitment: start"}
# {"level":"info","ts":"2021-07-26T12:30:47.775+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:1319","msg":"generate_data_commitment: finish"}
# {"level":"info","ts":"2021-07-26T12:31:47.335+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:1295","msg":"generate_data_commitment: start"}
# {"level":"info","ts":"2021-07-26T12:31:47.337+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:1319","msg":"generate_data_commitment: finish"}
# {"level":"info","ts":"2021-07-26T12:32:11.153+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:1295","msg":"generate_data_commitment: start"}
# {"level":"info","ts":"2021-07-26T12:32:11.154+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:1319","msg":"generate_data_commitment: finish"}
# {"level":"info","ts":"2021-07-26T12:35:11.681+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T12:35:11.681+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(87)"}
# {"level":"info","ts":"2021-07-26T12:37:49.702+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T12:37:49.702+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T12:37:50.763+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T12:37:50.763+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T12:52:25.680+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T12:52:25.680+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T13:15:08.419+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T13:15:08.419+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(84)"}
# {"level":"info","ts":"2021-07-26T13:34:07.845+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T13:34:07.845+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(91)"}
# {"level":"info","ts":"2021-07-26T13:54:06.871+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T13:54:06.871+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(93)"}
# {"level":"info","ts":"2021-07-26T14:13:37.900+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T14:13:37.900+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(92)"}
# {"level":"info","ts":"2021-07-26T14:34:01.270+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T14:34:01.272+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(94)"}
# {"level":"info","ts":"2021-07-26T14:53:38.137+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T14:53:38.139+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(95)"}
# {"level":"info","ts":"2021-07-26T15:11:09.700+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(89)"}
# {"level":"info","ts":"2021-07-26T15:11:09.701+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T15:11:09.891+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T15:11:09.893+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T15:13:30.198+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T15:13:30.198+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(96)"}
# {"level":"info","ts":"2021-07-26T15:14:30.953+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T15:14:30.953+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(100)"}
# {"level":"info","ts":"2021-07-26T15:22:22.494+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(83)"}
# {"level":"info","ts":"2021-07-26T15:22:22.495+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T15:30:19.823+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T15:30:19.823+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T15:30:19.907+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T15:30:19.907+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T15:32:37.475+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T15:32:37.475+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(101)"}
# {"level":"info","ts":"2021-07-26T15:43:07.288+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(87)"}
# {"level":"info","ts":"2021-07-26T15:43:07.289+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T15:49:20.211+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T15:49:20.211+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T15:49:20.290+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T15:49:20.291+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T16:07:12.501+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T16:07:12.502+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T16:22:07.132+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(84)"}
# {"level":"info","ts":"2021-07-26T16:22:07.132+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T16:22:07.317+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T16:22:07.317+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T16:38:12.095+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T16:38:12.095+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T16:41:36.473+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(91)"}
# {"level":"info","ts":"2021-07-26T16:41:36.473+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T16:41:36.605+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T16:41:36.606+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T16:57:27.554+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T16:57:27.554+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T17:04:03.675+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(93)"}
# {"level":"info","ts":"2021-07-26T17:04:03.675+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T17:04:03.837+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T17:04:03.837+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T17:12:39.275+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T17:12:39.275+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(97)"}
# {"level":"info","ts":"2021-07-26T17:21:10.756+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T17:21:10.756+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T17:23:50.851+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(92)"}
# {"level":"info","ts":"2021-07-26T17:23:50.851+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T17:23:51.060+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T17:23:51.061+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T17:32:39.450+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T17:32:39.450+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(85)"}
# {"level":"info","ts":"2021-07-26T17:41:22.833+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T17:41:22.833+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T17:44:37.444+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(94)"}
# {"level":"info","ts":"2021-07-26T17:44:37.444+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T17:44:37.610+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T17:44:37.611+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T17:52:37.881+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T17:52:37.881+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(82)"}
# {"level":"info","ts":"2021-07-26T18:01:42.782+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T18:01:42.782+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T18:09:28.783+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(95)"}
# {"level":"info","ts":"2021-07-26T18:09:28.784+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T18:09:28.917+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T18:09:28.917+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T18:19:38.399+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T18:19:38.399+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(86)"}
# {"level":"info","ts":"2021-07-26T18:26:23.763+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:320","msg":"seal_pre_commit_phase2:finish"}
# {"level":"info","ts":"2021-07-26T18:26:23.763+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:295","msg":"seal_pre_commit_phase2: finish"}
# {"level":"info","ts":"2021-07-26T18:33:59.977+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(100)"}
# {"level":"info","ts":"2021-07-26T18:33:59.977+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:196","msg":"seal_pre_commit_phase1:finish: SectorId(96)"}
# {"level":"info","ts":"2021-07-26T18:33:59.977+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T18:33:59.977+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:246","msg":"seal_pre_commit_phase1: finish"}
# {"level":"info","ts":"2021-07-26T18:34:00.313+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:264","msg":"seal_pre_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T18:34:00.314+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:211","msg":"seal_pre_commit_phase2:start"}
# {"level":"info","ts":"2021-07-26T18:39:09.239+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:213","msg":"seal_pre_commit_phase1: start"}
# {"level":"info","ts":"2021-07-26T18:39:09.239+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:68","msg":"seal_pre_commit_phase1:start: SectorId(88)"}

import os
import json

worker_log=os.path.abspath(".\\Calibnet\\pc1.log")
if (os.name != 'nt'): 
    sectors_log_dir=os.path.abspath("./worker.precommit.log")
    
from lotusops.speedtest.jsonformat import makeReport,printReports,analyzeReports

if __name__ == "__main__":
    reports = makeReport(worker_log,filters=["seal_pre_commit_phase1","SectorId"])
    printReports(reports)
    print("#################################")
    analyzeReports(reports)


# from __future__ import absolute_import

events=[
        "SectorStartCC",
        "SectorPacked",
        "SectorTicket",
        "SectorPreCommit1",
        "SectorPreCommit2",
        "SectorPreCommitBatch",
        "SectorPreCommitBatchSent",
        "SectorPreCommitLanded",
        "SectorSeedReady",
        "SectorCommitted",
        "SectorSubmitCommitAggregate",
        "SectorCommitAggregateSent",
        "SectorProving",
        "SectorFinalized",
        "SectorSealPreCommit1Failed",
        "SectorRetrySealPreCommit1",
]


import os
import sys

msg_help = "\n\
            \n======== lotusops =========\
            \n\
            \nDescription: \
            \n            A lotus-miner assistant tool for mainly processing job-abortion, sector-removal, pledging.\
            \n            It's a capsulation tool based on lotus-miner & shell scripts. Make sure lotus-miner is already installed.\
            \nUsage: \
            \n       [job-abortion] - to abort all sealing jobs with a certain keyword,\
            \n       lotusops abort <keyword>\
            \n       [sector-removal] - to remove all sectors with a certain keyword. \
            \n       lotusops remall <keyword>\
            \n       [multi-pledge] - to keep pledging with a certain interval until reaching to the indicated number in param. \
            \n       lotusops pledge <interval> <pledging-cnt>\
            \nExamples: \
            \n       [job-abortion]\
            \n       lotusops abort AP \
            \n       [sector-removal] \
            \n       lotusops rmall AP\
            \n       [multipledge] \
            \n       lotusops pledge 1m 28"

def lotusops():
    if len(sys.argv) < 3 or any(any(arg.startswith(mark) for mark in ["-h","--h","help"]) for arg in sys.argv): return msg_help
    if sys.argv[1] == "abort":
        execute(actioncode="abort",scripts="lotus-miner sealing jobs",filters=sys.argv[2].split("|"),excluded=len(sys.argv)>3)
    elif sys.argv[1] == "rmall":
        execute(actioncode="rmall",scripts="lotus-miner sectors list",filters=sys.argv[2].split("|"),excluded=len(sys.argv)>3)
    elif sys.argv[1] == "pledge" and len(sys.argv) > 3 and sys.argv[3].isdigit():
        execute(actioncode="pledge",scripts="lotus-miner sectors pledge",filters=sys.argv[2],excluded=sys.argv[3])
    else: return msg_help

import string

from lotusops.cli.lotuspledge import runscript
from lotusops.cli.lotuspledge import interpret

def execute(actioncode="abort",scripts="lotus-miner sealing jobs",filters=["AP"],excluded=False):
    # if pledge
    # pledge, lotus-miner sectors pledge, 1m, 28
    # for id in {1..28}; do lotus-miner sectors pledge; sleep 2m;done
    if actioncode == "pledge":
        interval=interpret(filters)
        try:
            cnt = int(excluded)
        except: cnt=1
        import time
        while cnt>0:
            print(runscript(scripts,isansible=False))
            cnt-=1
            time.sleep(interval)
        return
    # elif abort/rmall
    ids=[]
    try:
        if excluded:
            candidates=[line.split()[0] \
                        for line in runscript(scripts,isansible=False)[1:] \
                            if not any(filter in line for filter in filters) and \
                            all(c in string.hexdigits for c in line.split()[0])]
        else:
            candidates=[line.split()[0] \
                        for line in runscript(scripts,isansible=False)[1:]\
                            if all(filter in line for filter in filters)  and \
                                all(c in string.hexdigits for c in line.split()[0])]
        candidates=list(filter(None,candidates))
    except: return
    action = "lotus-miner sealing abort" if actioncode == "abort" else "lotus-miner sectors remove --really-do-it"
    for id in candidates:
        script=action+" %s"%id
        print(script)
        output = runscript(script,isansible=False)




import os
import sys

msg_help = "\n\
            \n======== lotusspeed =========\
            \n\
            \nDescription: \
            \n            A tool for observing time-consuming in a specific sealing stage,\
            \n            to analyze miner or worker's log single or batch\
            \nUsage: \
            \n       [*miner-log* using output of `lotus-miner sectors status --log <sector-id>`] \
            \n       lotusspeed miner\
            \n       [*worker-log* based on rust loging]   \
            \n       lotusspeed [sealing-stage] [filepath/dirpath] [format(json|txt)]]\
            \nExamples: \
            \n       [worker-log: precommit1]\
            \n       lotusspeed precommit1 p1.log \
            \n       [worker-log: precommit2]\
            \n       lotusspeed precommit2 p2.log \
            \n       [worker-log: commit2]\
            \n       lotusspeed commit2 c2.log\
            \n       [json format]\
            \n       lotusspeed commit2 c2.log json\
            \n       [worker-log: multiple files]\
            \n       lotusspeed commit2 c2.1.log c2.2.log"

from .__constants__ import msg_file_or_dir_not_found
from lotusops.speedtest.sectorstatus import AnalyzeSectorLogs
from lotusops.speedtest.jsonformat import analyzeFile
from lotusops.speedtest.textformat import analyzeFile as analyzeTxTFile

def lotusspeed():
    if len(sys.argv) < 2 or \
        any(any(arg.startswith(mark) for mark in ["-h","--h","help"]) for arg in sys.argv): return msg_help
    if "miner" == sys.argv[1]: return AnalyzeSectorLogs([] if len(sys.argv)==2 else sys.argv[2:])
    if not os.path.exists(sys.argv[2]) or not os.path.isfile(sys.argv[2]): return msg_file_or_dir_not_found
    filters = {
        "precommit1": ["seal_pre_commit_phase1"],
        "p1": ["seal_pre_commit_phase1"],
        "pc1": ["seal_pre_commit_phase1"],
        "precommit2": ["seal_pre_commit_phase2","filecoin_proofs"],
        "p2": ["seal_pre_commit_phase2","filecoin_proofs"],
        "pc2": ["seal_pre_commit_phase2","filecoin_proofs"],
        "commit2": ["seal_commit_phase2"],
        "c2": ["seal_commit_phase2"],
        "c": ["seal_commit_phase2"]
    }.get(sys.argv[1], ["seal_commit_phase2"])
    if len(sys.argv)>3 and any("json" in sys.argv[i] for i in range(3,len(sys.argv))):
        filters.append("SectorId")
        analyzeFile(os.path.abspath(sys.argv[2]),filters)
    else:
        analyzeTxTFile([os.path.abspath(sys.argv[i]) for i in range(2,len(sys.argv))],filters)



# {"level":"info","ts":"2021-07-26T15:10:04.756+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:374","msg":"seal_commit_phase2: start"}
# {"level":"info","ts":"2021-07-26T15:10:04.836+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:459","msg":"seal_commit_phase2:start: SectorId(81)"}
# {"level":"info","ts":"2021-07-26T15:10:04.836+0800","logger":"filecoin_proofs::caches","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/caches.rs:49","msg":"trying parameters memory cache for: STACKED[34359738368]"}
# cat /filecoin/worker.log|grep seal_commit_phase2
# {"level":"info","ts":"2021-07-26T15:10:04.836+0800","logger":"filecoin_proofs::caches","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/caches.rs:54","msg":"found params in memory cache for STACKED[34359738368]"}
# {"level":"info","ts":"2021-07-26T15:10:04.836+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:488","msg":"got groth params (34359738368) while sealing"}
# {"level":"info","ts":"2021-07-26T15:10:04.836+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:509","msg":"snark_proof:start"}
################# 2.5min{ 
# {"level":"info","ts":"2021-07-26T15:10:04.848+0800","logger":"bellperson::groth16::prover","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/groth16/prover.rs:276","msg":"Bellperson 0.14.2 is being used!"}
# {"level":"info","ts":"2021-07-26T15:12:34.490+0800","logger":"bellperson::groth16::prover","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/groth16/prover.rs:561","msg":"starting proof timer"}
################# } GPU is available for FFT!
# {"level":"info","ts":"2021-07-26T15:12:36.907+0800","logger":"bellperson::gpu::locks","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/gpu/locks.rs:157","msg":"GPU is available for FFT!"}
# {"level":"debug","ts":"2021-07-26T15:12:36.907+0800","logger":"bellperson::gpu::locks","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/gpu/locks.rs:20","msg":"Acquiring GPU lock at "/tmp/bellman.gpu.lock" ..."}
# {"level":"debug","ts":"2021-07-26T15:12:36.907+0800","logger":"bellperson::gpu::locks","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/gpu/locks.rs:24","msg":"GPU lock acquired!"}
# {"level":"info","ts":"2021-07-26T15:12:37.018+0800","logger":"bellperson::gpu::fft","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/gpu/fft.rs:47","msg":"FFT: 1 working device(s) selected."}
# {"level":"info","ts":"2021-07-26T15:12:37.018+0800","logger":"bellperson::gpu::fft","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/gpu/fft.rs:48","msg":"FFT: Device 0: GeForce RTX 2080 Ti"}
################ 3.5min{ GPU FFT kernel instantiated!
# {"level":"info","ts":"2021-07-26T15:12:37.018+0800","logger":"bellperson::domain","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/domain.rs:568","msg":"GPU FFT kernel instantiated!"}
# {"level":"debug","ts":"2021-07-26T15:16:15.906+0800","logger":"bellperson::gpu::locks","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/gpu/locks.rs:31","msg":"GPU lock released!"}
################ } GPU is available for Multiexp!
# {"level":"info","ts":"2021-07-26T15:16:20.413+0800","logger":"bellperson::gpu::locks","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/gpu/locks.rs:158","msg":"GPU is available for Multiexp!"}
# {"level":"debug","ts":"2021-07-26T15:16:20.413+0800","logger":"bellperson::gpu::locks","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/gpu/locks.rs:20","msg":"Acquiring GPU lock at "/tmp/bellman.gpu.lock" ..."}
# {"level":"debug","ts":"2021-07-26T15:16:20.413+0800","logger":"bellperson::gpu::locks","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/gpu/locks.rs:24","msg":"GPU lock acquired!"}
# {"level":"info","ts":"2021-07-26T15:16:20.624+0800","logger":"bellperson::gpu::multiexp","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/gpu/multiexp.rs:246","msg":"Multiexp: 2 working device(s) selected. (CPU utilization: 0)"}
# {"level":"info","ts":"2021-07-26T15:16:20.624+0800","logger":"bellperson::gpu::multiexp","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/gpu/multiexp.rs:252","msg":"Multiexp: Device 0: GeForce RTX 2080 Ti (Chunk-size: 7488999)"}
# {"level":"info","ts":"2021-07-26T15:16:20.624+0800","logger":"bellperson::gpu::multiexp","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/gpu/multiexp.rs:252","msg":"Multiexp: Device 1: GeForce RTX 2080 Ti (Chunk-size: 7488999)"}
################ 13min{ GPU Multiexp kernel instantiated!
# {"level":"info","ts":"2021-07-26T15:16:20.624+0800","logger":"bellperson::multiexp","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/multiexp.rs:402","msg":"GPU Multiexp kernel instantiated!"}
# {"level":"debug","ts":"2021-07-26T15:29:18.412+0800","logger":"bellperson::gpu::locks","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/gpu/locks.rs:31","msg":"GPU lock released!"}
################ }
# {"level":"info","ts":"2021-07-26T15:29:18.436+0800","logger":"bellperson::groth16::prover","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/bellperson-0.14.2/src/groth16/prover.rs:522","msg":"prover time: 1003.945903624s"}
# {"level":"info","ts":"2021-07-26T15:29:21.069+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:517","msg":"snark_proof:finish"}
# {"level":"info","ts":"2021-07-26T15:29:21.069+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:927","msg":"verify_seal:start: SectorId(81)"}
# {"level":"info","ts":"2021-07-26T15:29:21.069+0800","logger":"filecoin_proofs::caches","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/caches.rs:49","msg":"trying parameters memory cache for: STACKED[34359738368]-verifying-key"}
# {"level":"info","ts":"2021-07-26T15:29:21.069+0800","logger":"filecoin_proofs::caches","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/caches.rs:54","msg":"found params in memory cache for STACKED[34359738368]-verifying-key"}
# {"level":"info","ts":"2021-07-26T15:29:21.069+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:970","msg":"got verifying key (34359738368) while verifying seal"}
# {"level":"info","ts":"2021-07-26T15:29:21.083+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:995","msg":"verify_seal:finish: SectorId(81)"}
# {"level":"info","ts":"2021-07-26T15:29:21.083+0800","logger":"filecoin_proofs::api::seal","caller":"/root/.cargo/registry/src/github.com-1ecc6299db9ec823/filecoin-proofs-8.0.2/src/api/seal.rs:543","msg":"seal_commit_phase2:finish: SectorId(81)"}
# {"level":"info","ts":"2021-07-26T15:29:21.083+0800","logger":"filcrypto::proofs::api","caller":"src/proofs/api.rs:400","msg":"seal_commit_phase2: finish"}

import os
import json

worker_log=os.path.abspath(".\\Calibnet\\worker.commit.log")
if (os.name != 'nt'): 
    sectors_log_dir=os.path.abspath("./worker.commit.log")
    
def msg2sectorId(msg):
    s = msg.find("(")
    e = msg.find(")")
    import string
    if s < 0 or e < 0 or e < s or \
       any(c not in string.digits for c in msg[s+1:e]): return -1
    return int(msg[s+1:e])

# in:  2021-06-21 23:52:25+0800
#      2021-06-21 23:52:25.442
#      2021-06-21 23:52:25 +0800 CST:
# out: datetime.datetime(2021, 6, 21, 23, 52, 25)
import datetime
def string2datetime(st):
    if " +0800 CST:"in st: return datetime.datetime.strptime(st, "%Y-%m-%d %H:%M:%S +0800 CST:")
    if "+800" in st: return datetime.datetime.strptime(st, "%Y-%m-%dT%H:%M:%S.%f+0800")
    return datetime.datetime.strptime(st, "%Y-%m-%dT%H:%M:%S.%f")

def makeReport(logfile,filters=["seal_commit_phase2","SectorId"]):
    sectors={}
    with open(logfile,'r') as f:
        lines=f.readlines()
        lastfinish=-1
        for line in lines:
            log=line.rstrip("\n")
            try: jslog=json.loads(log)
            except: continue
            if "msg" not in jslog.keys() or \
               any(filstr not in jslog["msg"] for filstr in  filters): continue
            
            id=msg2sectorId(jslog["msg"])
            ts=string2datetime(jslog["ts"])
            if id not in sectors.keys(): sectors[id]={}
            if "start" in jslog["msg"]: 
                sectors[id]["start"] = ts
                sectors[id]["interval"]=sectors[id]["start"] - lastfinish if lastfinish != -1 and sectors[id]["start"] > lastfinish else datetime.timedelta(0)
                sectors[id]["duration"] = datetime.timedelta(0)
                sectors[id]["finish"] = ts
            if "finish" in jslog["msg"]: 
                sectors[id]["finish"] = ts
                sectors[id]["duration"] = sectors[id]["finish"] - sectors[id]["start"] if sectors[id]["finish"] > sectors[id]["start"] else datetime.timedelta(0)
                lastfinish = ts
    return sectors

from lotusops.speedtest.sectorstatus import strfdelta

def analyzeReports(reports,stages=["duration","interval"]):
    print("*******************************************")
    for stage in stages:
        mean_=datetime.timedelta(0)
        max_=datetime.timedelta(0)
        min_=datetime.timedelta(days=10)
        mean_=datetime.timedelta(0)
        ## initialize
        sum=datetime.timedelta(0)
        cnt=0
        for id in reports.keys():
            report=reports[id]
            if stage not in report.keys(): continue
            if report[stage] == datetime.timedelta(0): continue
            if report[stage] > max_: max_=report[stage]
            if report[stage] < min_: min_=report[stage]
            sum+= report[stage]
            cnt+=1
        if cnt == 0: break
        mean_=sum/cnt
        print("MIN({0}) ---{1}".format(stage,strfdelta(min_)))
        print("MAX({0}) ---{1}".format(stage,strfdelta(max_)))
        print("MEAN({0})---{1}".format(stage,strfdelta(mean_)))
        print("*******************************************")
               
def printReports(reports):
    for id in reports.keys():
        print("SectorId({0})-{1}-({2})-({3})".format(id,strfdelta(reports[id]["duration"]),reports[id]["start"].strftime("%Y-%m-%d %H:%M:%S"),reports[id]["finish"].strftime("%Y-%m-%d %H:%M:%S")))
    print("#################################")
    for id in reports.keys():
        print("SectorId({0})-{1}".format(id,reports[id]["interval"]))

def analyzeFile(filepath,filters,isdetailed=False):
    reports = makeReport(filepath,filters=["seal_commit_phase2","SectorId"])
    if isdetailed: printReports(reports)
    print("#################################")
    analyzeReports(reports)    

if __name__ == "__main__":
    analyzeFile(worker_log)
    
# 2021-07-24T22:19:31.744 INFO filcrypto::proofs::api > seal_commit_phase2: start
# 2021-07-24T22:19:31.858 INFO filecoin_proofs::api::seal > seal_commit_phase2:start: SectorId(4)
# 2021-07-24T22:19:31.859 INFO filecoin_proofs::caches > trying parameters memory cache for: STACKED[34359738368]
# 2021-07-24T22:19:31.859 INFO filecoin_proofs::caches > found params in memory cache for STACKED[34359738368]
# 2021-07-24T22:19:31.859 INFO filecoin_proofs::api::seal > got groth params (34359738368) while sealing
# 2021-07-24T22:19:31.859 INFO filecoin_proofs::api::seal > snark_proof:start
####################### 3min{ bellperson::groth16
# 2021-07-24T22:19:31.879 INFO bellperson::groth16::prover > Bellperson 0.14.2 is being used!
# 2021-07-24T22:22:29.014 INFO bellperson::groth16::prover > starting proof timer
####################### } GPU is available for FFT!
# 2021-07-24T22:22:38.887 INFO bellperson::gpu::locks > GPU is available for FFT!
# 2021-07-24T22:22:38.887 DEBUG bellperson::gpu::locks > Acquiring GPU lock at "/filecoin/lotustmp/bellman.gpu.lock" ...
# 2021-07-24T22:22:38.887 DEBUG bellperson::gpu::locks > GPU lock acquired!
# 2021-07-24T22:22:39.042 INFO bellperson::gpu::fft > FFT: 1 working device(s) selected.
# 2021-07-24T22:22:39.042 INFO bellperson::gpu::fft > FFT: Device 0: GeForce RTX 2080 Ti
######################  5.5min{ GPU FFT kernel instantiated!
# 2021-07-24T22:22:39.042 INFO bellperson::domain > GPU FFT kernel instantiated!
# 2021-07-24T22:28:04.580 DEBUG bellperson::gpu::locks > GPU lock released!
###################### } GPU is available for Multiexp!
# 2021-07-24T22:28:14.442 INFO bellperson::gpu::locks > GPU is available for Multiexp!
# 2021-07-24T22:28:14.442 DEBUG bellperson::gpu::locks > Acquiring GPU lock at "/filecoin/lotustmp/bellman.gpu.lock" ...
# 2021-07-24T22:28:14.442 DEBUG bellperson::gpu::locks > GPU lock acquired!
# 2021-07-24T22:28:15.395 INFO bellperson::gpu::multiexp > Multiexp: 6 working device(s) selected. (CPU utilization: 0)
# 2021-07-24T22:28:15.395 INFO bellperson::gpu::multiexp > Multiexp: Device 0: GeForce RTX 2080 Ti (Chunk-size: 7488999)
# 2021-07-24T22:28:15.395 INFO bellperson::gpu::multiexp > Multiexp: Device 1: GeForce RTX 2080 Ti (Chunk-size: 7488999)
# 2021-07-24T22:28:15.395 INFO bellperson::gpu::multiexp > Multiexp: Device 2: GeForce RTX 2080 Ti (Chunk-size: 7488999)
# 2021-07-24T22:28:15.395 INFO bellperson::gpu::multiexp > Multiexp: Device 3: GeForce RTX 2080 Ti (Chunk-size: 7488999)
# 2021-07-24T22:28:15.395 INFO bellperson::gpu::multiexp > Multiexp: Device 4: GeForce RTX 2080 Ti (Chunk-size: 7488999)
# 2021-07-24T22:28:15.395 INFO bellperson::gpu::multiexp > Multiexp: Device 5: GeForce RTX 2080 Ti (Chunk-size: 7488999)
######################## 5.5min{  GPU Multiexp kernel instantiated!
# 2021-07-24T22:28:15.395 INFO bellperson::multiexp > GPU Multiexp kernel instantiated!
# 2021-07-24T22:36:41.818 WARN bellperson::gpu::locks > GPU acquired by a high priority process! Freeing up Multiexp kernels...
######################## }
# 2021-07-24T22:36:42.089 DEBUG bellperson::gpu::locks > GPU lock released!
# 2021-07-24T22:36:42.089 INFO bellperson::gpu::locks > GPU is available for Multiexp!
# 2021-07-24T22:36:42.089 DEBUG bellperson::gpu::locks > Acquiring GPU lock at "/filecoin/lotustmp/bellman.gpu.lock" ...
# 2021-07-24T22:36:42.089 DEBUG bellperson::gpu::locks > GPU lock acquired!
# 2021-07-24T22:36:42.835 INFO bellperson::gpu::multiexp > Multiexp: 6 working device(s) selected. (CPU utilization: 0)
# 2021-07-24T22:36:42.835 INFO bellperson::gpu::multiexp > Multiexp: Device 0: GeForce RTX 2080 Ti (Chunk-size: 7488999)
# 2021-07-24T22:36:42.835 INFO bellperson::gpu::multiexp > Multiexp: Device 1: GeForce RTX 2080 Ti (Chunk-size: 7488999)
# 2021-07-24T22:36:42.835 INFO bellperson::gpu::multiexp > Multiexp: Device 2: GeForce RTX 2080 Ti (Chunk-size: 7488999)
# 2021-07-24T22:36:42.835 INFO bellperson::gpu::multiexp > Multiexp: Device 3: GeForce RTX 2080 Ti (Chunk-size: 7488999)
# 2021-07-24T22:36:42.835 INFO bellperson::gpu::multiexp > Multiexp: Device 4: GeForce RTX 2080 Ti (Chunk-size: 7488999)
# 2021-07-24T22:36:42.835 INFO bellperson::gpu::multiexp > Multiexp: Device 5: GeForce RTX 2080 Ti (Chunk-size: 7488999)
######################## 6.5min{ GPU Multiexp kernel instantiated!
# 2021-07-24T22:36:42.835 INFO bellperson::multiexp > GPU Multiexp kernel instantiated!
# 2021-07-24T22:43:07.124 DEBUG bellperson::gpu::locks > GPU lock released!
######################## }
# 2021-07-24T22:43:07.147 INFO bellperson::groth16::prover > prover time: 1238.133344429s
# 2021-07-24T22:43:09.454 INFO filecoin_proofs::api::seal > snark_proof:finish
# 2021-07-24T22:43:09.455 INFO filecoin_proofs::api::seal > verify_seal:start: SectorId(4)
# 2021-07-24T22:43:09.455 INFO filecoin_proofs::caches > trying parameters memory cache for: STACKED[34359738368]-verifying-key
# 2021-07-24T22:43:09.455 INFO filecoin_proofs::caches > found params in memory cache for STACKED[34359738368]-verifying-key
# 2021-07-24T22:43:09.455 INFO filecoin_proofs::api::seal > got verifying key (34359738368) while verifying seal
# 2021-07-24T22:43:09.471 INFO filecoin_proofs::api::seal > verify_seal:finish: SectorId(4)
# 2021-07-24T22:43:09.471 INFO filecoin_proofs::api::seal > seal_commit_phase2:finish: SectorId(4)
# 2021-07-24T22:43:09.471 INFO filcrypto::proofs::api > seal_commit_phase2: finish




import toml

lotusops_setting_filepath = "/etc/lotusops/config/setting.toml"

def showSettings():
    print("The configuration file is located in %s.\nThe modification of it's content will affect this tool's operations.\n"%lotusops_setting_filepath)
    with open(lotusops_setting_filepath,"r") as f:
        config = toml.loads(f.read())
        config_string = toml.dumps(config)
        print(config_string)

def loadSetting():
    config={}
    with open(lotusops_setting_filepath,"r") as f:
        config = toml.loads(f.read())
    return config

def saveSettings(new_config):
    config_string = toml.dumps(new_config)
    with open(lotusops_setting_filepath,"w") as f:
        f.write(config_string)

def changeSetting(k,sk,v):
    with open(lotusops_setting_filepath,"rw") as f:
        config = toml.loads(f.read())
        if k not in config.keys(): config[k] = {}
        config[k][sk] = v
        f.write(config)


entry_points = [
                # log analytics      
                "lotusspeed = lotusops.cli.speedtest:lotusspeed",
                # lotusops - rmall, abort,
                "lotusops = lotusops.cli.lotusops:lotusops",
                #  autopledge
                "lotuspledge = lotusops.cli.lotuspledge:lotuspledge",
                #  show configuration & commands
                "lotusopsabout = lotusops.cli.lotusopsabout:lotusopsabout",                
]

commands = [cmd.strip().replace(" ",'').split('=')[0] for cmd in entry_points]


import os
import sys

msg_help = "\n\
            \n======== lotusopsabout =========\
            \n\
            \nDescription: \
            \n             Show avaiable commands & setting for lotus.\
            \nUsage: \
            \n       lotusopsabout settings\
            \n       lotusopsabout commands"

from lotusops.config.setting import showSettings

def lotusopsabout():
        if len(sys.argv) == 1 or any(any(arg.startswith(mark) for mark in ["-h","--h","help"]) for arg in sys.argv): return msg_help
        if sys.argv[1] == "settings": showSettings();return
        if sys.argv[1] == "commands": print("\n".join(["lotusops","lotusspeed","lotuspledge","lotusabout"]));return
        return msg_help






from lotusops.speedtest.jsonformat import string2datetime,msg2sectorId
# in:  2021-07-26T11:32:54.442
# out: datetime.datetime(2021, 7, 26, 11, 32, 54)
# import datetime
# def string2datetime(st):
#     return datetime.datetime.strptime(st, "%Y-%m-%dT%H:%M:%S.%f")

from lotusops.speedtest.sectorstatus import strfdelta
import datetime

def analyzeFile(filepaths,filters):
    total_cnt,total_sum=0,datetime.timedelta(0)
    start,finish=string2datetime("2089-01-01T00:00:00.000"),string2datetime("1989-01-01T00:00:00.000")
    for filepath in filepaths:
        sector,sectors={},{}
        with open(filepath,"r") as f:
            lines=f.readlines()
            for line in lines:
                if any(filter not in line for filter in filters): continue
                # structed=line.strip("\t")
                msg = line.rstrip("\n")
                if not any(mark in msg for mark in ["start","finish"]): continue
                time=string2datetime(msg.split()[0])
                # in case of precommit2
                if any("seal_pre_commit_phase2" in filter for filter in filters):
                    if "start" in msg: sector["start"]=time; start=start if time > start else time;continue
                    if "finish" in msg: sector["finish"]=time; finish=finish if time < finish else time
                    if all(key in sector for key in ["start","finish"]): 
                        sector["period"]=sector["finish"]-sector["start"]
                        print("DURATION:{0}    START:{1} FINISH:{2}".format(strfdelta(sector["period"]),sector["start"].strftime("%Y-%m-%d %H:%M:%S"),sector["finish"].strftime("%Y-%m-%d %H:%M:%S")))
                        sectors[sector["start"]]=sector
                    sector={};continue
                # in case of precommit1, commit2
                id = msg2sectorId(msg)
                if id < 0: continue
                if id not in sectors.keys(): sectors[id]={}
                if "start" in msg: sectors[id]["start"]=time; start=start if time > start else time; continue
                if "finish" in msg: sectors[id]["finish"]=time; finish=finish if time < finish else time
                if all(key in sectors[id] for key in ["start","finish"]): 
                    sectors[id]["period"]=sectors[id]["finish"]-sectors[id]["start"]
                    print("SectorId({0})- DURATION:{1}    START:{2} FINISH:{3}".format(id,strfdelta(sectors[id]["period"]),sectors[id]["start"].strftime("%Y-%m-%d %H:%M:%S"),sectors[id]["finish"].strftime("%Y-%m-%d %H:%M:%S")))
            # analyze
            
            ## max, min, mean
            max_=datetime.timedelta(0)
            min_=datetime.timedelta(days=10)
            mean_=datetime.timedelta(0)
            ## initialize
            sum=datetime.timedelta(0)
            cnt=0
            for k,v in sectors.items():
                if "period" not in v.keys(): continue
                cnt+=1
                if v["period"] < min_: min_= v["period"]
                if v["period"] > max_: max_= v["period"]
                sum+=v["period"]
            mean_=sum/cnt if cnt!=0 else sum
            total_cnt+=cnt
            total_sum+=sum
            print("*******************************************")  
            print("MIN:  {0}".format(strfdelta(min_)))
            print("MAX:  {0}".format(strfdelta(max_)))
            print("MEAN: {0}".format(strfdelta(mean_)))
            print("*******************************************")
    # speed calculation
    if total_cnt == 0: return
    duration=(finish-start).total_seconds()/(24*3600.0)#days
    meantime=(total_sum/total_cnt).total_seconds()/(24*3600.0)#days
    sector_size = 32#GiB
    _as_terabyte = sector_size/1000.0
    dspeed = total_cnt*_as_terabyte/duration
    mspeed = len(filepaths) *_as_terabyte/meantime
    print("SectorSize: 32GiB, SectorCnt: {0}, MeanTime: {1}".\
        format(total_cnt,\
              "%.1f hours"%(meantime*24) if any("seal_pre_commit_phase1" in filter for filter in filters) else \
              "%.1f mins"%(meantime*24*60)))
    print("DURATION:{0} START: {1}, FINISH: {2}".\
        format(strfdelta(finish-start),\
            start.strftime("%Y-%m-%d %H:%M:%S"),\
            finish.strftime("%Y-%m-%d %H:%M:%S")))
    if any("seal_pre_commit_phase1" in filter for filter in filters):
        print("MINING RATE: {0}".format("%.1fTiB/D"%dspeed))
    else:
        print("MINING RATE: {0}|{1}TiB/D".format("%.1f"%dspeed,"%.1f"%mspeed))

